package java.nio.file;

import java.io.InputStream;
import java.io.IOException;
import java.nio.file.attribute.BasicFileAttributeView;
import java.nio.file.attribute.FileAttribute;
import java.nio.file.attribute.BasicFileAttributes;

class CopyMoveHelper
{
    private CopyMoveHelper() {
    }
    
    private static CopyOption[] convertMoveToCopyOptions(CopyOption... array) throws AtomicMoveNotSupportedException {
        int length = array.length;
        CopyOption[] array2 = new CopyOption[length + 2];
        int i = 0;
        while (i < length) {
            CopyOption copyOption = array[i];
            if (copyOption == StandardCopyOption.ATOMIC_MOVE) {
                throw new AtomicMoveNotSupportedException(null, null, "Atomic move between providers is not supported");
            }
            array2[i] = copyOption;
            ++i;
        }
        array2[length] = LinkOption.NOFOLLOW_LINKS;
        array2[length + 1] = StandardCopyOption.COPY_ATTRIBUTES;
        return array2;
    }
    
    static void copyToForeignTarget(Path path, Path path2, CopyOption... array) throws IOException {
        CopyMoveHelper.CopyOptions parse = CopyMoveHelper.CopyOptions.parse(array);
        BasicFileAttributes attributes = Files.readAttributes(path, BasicFileAttributes.class, parse.followLinks ? new LinkOption[0] : new LinkOption[] { LinkOption.NOFOLLOW_LINKS });
        if (attributes.isSymbolicLink()) {
            throw new IOException("Copying of symbolic links not supported");
        }
        if (parse.replaceExisting) {
            Files.deleteIfExists(path2);
        }
        else if (Files.exists(path2, new LinkOption[0])) {
            throw new FileAlreadyExistsException(path2.toString());
        }
        if (attributes.isDirectory()) {
            Files.createDirectory(path2, new FileAttribute[0]);
        }
        else {
            try (final InputStream inputStream = Files.newInputStream(path, new OpenOption[0])) {
                Files.copy(inputStream, path2, new CopyOption[0]);
            }
        }
        if (parse.copyAttributes) {
            BasicFileAttributeView basicFileAttributeView = (BasicFileAttributeView)Files.getFileAttributeView(path2, BasicFileAttributeView.class, new LinkOption[0]);
            try {
                basicFileAttributeView.setTimes(attributes.lastModifiedTime(), attributes.lastAccessTime(), attributes.creationTime());
            }
            catch (Throwable t3) {
                try {
                    Files.delete(path2);
                }
                catch (Throwable exception3) {
                    t3.addSuppressed(exception3);
                }
                throw t3;
            }
        }
        return;
    }
    
    static void moveToForeignTarget(Path path, Path path2, CopyOption... array) throws IOException {
        copyToForeignTarget(path, path2, convertMoveToCopyOptions(array));
        Files.delete(path);
    }
    
    private static class CopyOptions
    {
        boolean replaceExisting;
        boolean copyAttributes;
        boolean followLinks;
        
        private CopyOptions() {
            this.replaceExisting = false;
            this.copyAttributes = false;
            this.followLinks = true;
        }
        
        static CopyMoveHelper.CopyOptions parse(CopyOption... array) {
            CopyMoveHelper.CopyOptions copyOptions = new CopyMoveHelper.CopyOptions();
            for (CopyOption obj : array) {
                if (obj == StandardCopyOption.REPLACE_EXISTING) {
                    copyOptions.replaceExisting = true;
                }
                else if (obj == LinkOption.NOFOLLOW_LINKS) {
                    copyOptions.followLinks = false;
                }
                else if (obj == StandardCopyOption.COPY_ATTRIBUTES) {
                    copyOptions.copyAttributes = true;
                }
                else {
                    if (obj == null) {
                        throw new NullPointerException();
                    }
                    throw new UnsupportedOperationException("'" + obj + "' is not a recognized copy option");
                }
            }
            return copyOptions;
        }
    }
}
